# Generated by Django 4.2.13 on 2024-06-04 11:12

import cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cradmin_temporaryfileuploadstore', '0004_auto_20151017_1947'),
    ]

    operations = [
        migrations.AlterField(
            model_name='temporaryfile',
            name='file',
            field=models.FileField(storage=cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models.get_temporary_file_storage, upload_to=cradmin_legacy.apps.cradmin_temporaryfileuploadstore.models.temporary_file_upload_to),
        ),
    ]
