
.body-side-padding() {
  //padding-left: @cradmin-bodypadding-xs;
  //padding-right: @cradmin-bodypadding-xs;
  //
  //@media (min-width: @screen-sm-min) {
  //  padding-left: @cradmin-bodypadding;
  //  padding-right: @cradmin-bodypadding;
  //}
}

// Used by buttons, but we also need to apply this via a css class
// on file fields - so a mixin!
.btn-default-hoverstyle-mixin() {
  background: @btn-default-bg;
  border-color: @btn-default-border;
  color: @btn-default-color;
  @media (min-width: @screen-sm-min) {
    background: @cradmin-btn-default-hover-bg;
    border-color: @cradmin-btn-default-hover-border;
    color: @cradmin-btn-default-hover-color;
  }
}


.style-scrollbar(@width, @track-bg, @thumb-bg, @thumb-hover-bg) {
    // IE
  scrollbar-face-color: @thumb-bg;
  scrollbar-arrow-color: @thumb-bg;
  scrollbar-track-color: @track-bg;
  scrollbar-highlight-color: @thumb-bg;
  scrollbar-3dlight-color: @track-bg;
  scrollbar-dark-shadow-color: @track-bg;
  scrollbar-shadow-color: @track-bg;

  //// The entire scrollbar - this MUST be included (all the other scrollbar settings are ignored without this
  &::-webkit-scrollbar {
    width: @width;
  }

  // The background of track where the scrollbar slides
  &::-webkit-scrollbar-track {
    background-color: @track-bg;
  }

  // The box sliding up and down the track
  &::-webkit-scrollbar-thumb {
    background-color: @thumb-bg;
    &:hover {
      background-color: @thumb-hover-bg;
    }
  }

  // The up/down buttons
  &::-webkit-scrollbar-button {
    display: none;
  }
}


.css3-transform(@properties) {
  -webkit-transform: @properties;
     -moz-transform: @properties;
          transform: @properties;
}

.css3-transition(@properties) {
  -webkit-transition: @properties;
     -moz-transition: @properties;
          transition: @properties;
}


.cradmin-sr-only() {
  position: absolute;
  width: 1px;
  height: 1px;
  margin: -1px;
  padding: 0;
  overflow: hidden;
  clip: rect(0,0,0,0);
  border: 0;
}


.cradminFormControl() {
  border: 1px solid @cradmin-form-field-inactive-border-color;
  border-radius: 4px;
  outline: none;
  box-shadow: none;
  //padding: 10px 12px;
  background: @cradmin-form-field-inactive-bg;
  &:focus {
    background: @cradmin-form-field-focus-bg;
    border-color: @cradmin-form-field-focus-border-color;
    box-shadow: @cradmin-form-field-focus-box-shadow;
  }
}
