nav#cradmin_legacy_mainmenu.cradmin-legacy-menu {
  .cradminLegacyMenuVertical(
    @cradmin-topmenu-theme-mainmenu-desktop-height,
    @cradmin-topmenu-theme-mainmenu-desktop-item-padding,
    @cradmin-menu-mobiletoggle-height);
}
