.cradmin-legacy-multiselect2-itemvalue {
  .cradmin-legacy-multiselect2-itemvalue-buttons {
    margin-top: 8px;
    margin-bottom: 0;
    //.cradmin-legacy-multiselect2-itemvalue-button {
    //}
  }
  .cradmin-legacy-multiselect2-itemvalue-selectedsummary {
    display: none;
  }
}

.cradmin-legacy-multiselect2-item-wrapper-selected {
  .cradmin-legacy-multiselect2-itemvalue {
    .cradmin-legacy-multiselect2-itemvalue-selectedsummary {
      display: block;
      font-size: @font-size-small;
      color: @text-muted;
      .cradmin-legacy-multiselect2-itemvalue-selectedsummary-prefix-icon {
        color: @state-success-text;
      }
    }
    .cradmin-legacy-multiselect2-itemvalue-details {
      display: none;
    }
    .cradmin-legacy-multiselect2-itemvalue-buttons {
      display: none;
    }
  }
}
