@import "../cradmin_base/all";
@import "variables";
@import "menu";


body.cradmin-legacy-formview-page {
  .cradmin-legacy-page-header-outer {
    text-align: center;
  }
  ul#cradmin_legacy_messages {
    margin-left: auto;
    margin-right: auto;
  }
}
