.cradmin-legacy-focusedpage-wrapper {
  width: 100%;
  height: 100%;

  @media (min-width: @screen-sm-min) {
    margin-top: 110px;
    margin-left: auto;
    margin-right: auto;
    height: auto;
    width: 600px;
  }

  &.cradmin-legacy-focusedpage-wrapper-narrow {
    @media (min-width: @screen-sm-min) {
      width: 400px;
    }
  }

  .cradmin-legacy-focusedpage-wrapper-inner {
    background: #fff;
    padding: 24px 30px;
    text-align: left;
    margin-bottom: 30px;

    .page-header {
      background: none;
      color: @text-color;
      padding: 0 0 25px 0;
      border: none;
      margin: 0;
      h1 {
        margin: 0 0 5px 0;
        font-size: ceil(@font-size-base * 1.6);
        font-weight: bold;
      }
      p:last-child {
        margin: 0;
      }
    }

    @media (min-width: @screen-sm-min) {
      border: 1px solid #ccc;
    }

    .page-header {

    }
    .cradmin-legacy-focusedpage-content {

    }
  }
}
