ol.cradmin-legacy-listfilter-filterlist {
  list-style: none;
  margin: 0;
  padding: 0;
  li {
    margin: 0;
    padding: 0;
  }

  &.cradmin-legacy-listfilter-filterlist-vertical {
    margin-bottom: 20px;
    @media (min-width: @screen-sm-min) {
      margin-bottom: 0;
    }
    li {
      margin-top: 18px;
      &:first-child {
        margin-top: 0;
      }
    }
  }

  &.cradmin-legacy-listfilter-filterlist-horizontal {
    margin-bottom: 10px;
    text-align: center;
    li {
      display: inline-block;
      margin-right: 10px;
      text-align: left;
      &:last-child {
        margin-right: 0;
      }
    }
  }
}
