.container {
  .body-side-padding();
}

.container-focus {
  padding-top: 40px;
  padding-bottom: 40px;
  background: #fff;
}

.cradmin-legacy-hide-menu {
  #cradmin_legacy_contentwrapper {
    margin-left: auto;
    margin-right: auto;
  }
}
