.cradmin-legacy-listfilter-target-loading,
.cradmin-legacy-listfilter-target-loaderror {
  position: relative;

  .cradmin-legacy-listfilter-message {
    position: absolute;
    top: 0;
    left: 0;
    right: 0;
    padding: 15px;
    text-align: center;

    .cradmin-legacy-listfilter-message-loadspinner {
      margin-bottom: 10px;
      font-size: ceil(@font-size-base + 6px);
    }
    .cradmin-legacy-listfilter-message-text {
      font-weight: bold;
      display: block;
    }

    @media (min-width: @screen-sm-min) {
      left: 50%;
      margin-left: -150px;
      width: 300px;
      border-radius: @alert-border-radius;
    }

    &.cradmin-legacy-listfilter-message-loading {
      background: @alert-info-bg;
      border: 1px solid @alert-info-border;
      color: @alert-info-text;
    }

    &.cradmin-legacy-listfilter-message-error {
      background: @alert-danger-bg;
      border: 1px solid @alert-danger-border;
      color: @alert-danger-text;
    }
  }
}
