.cradmin-legacy-multiselect2-selected-item {
  display: none;
}

.cradmin-legacy-multiselect2-target {
  .cradmin-legacy-multiselect2-target-title {
    font-size: @font-size-base;
    font-weight: bold;
    margin: 20px 0 10px 0;
    @media (min-width: @screen-sm-min) {
      margin-top: 0;
      min-height: 28px;
    }
  }
  .cradmin-legacy-multiselect2-target-selected-items {
    .cradmin-legacy-multiselect2-target-selected-item {
      background: @cradmin-focus-bg;
      padding: @cradmin-multiselect2-target-selected-item-padding;
      margin-top: 5px;
      &:first-child {
        margin-top: 0;
      }
      .cradmin-legacy-multiselect2-target-selected-item-title {
        font-size: @font-size-base;
        //margin-top: 10px;
        margin: 0;
        padding: 0;
      }
      .cradmin-legacy-multiselect2-target-selected-item-description {
        margin-top: 10px;
        font-size: @font-size-small;
      }
      .cradmin-legacy-multiselect2-target-selected-item-deselectbutton {
        margin-top: 10px;
      }
    }
  }
  .cradmin-legacy-multiselect2-target-formfields {
    .form-group {
      margin-top: 20px;
      margin-bottom: 20px;
      .control-label  {
        margin-bottom: 0;
      }
      .controls {
        margin-top: 7px;
        .checkbox label {
          font-weight: normal;
          margin-top: 10px;
          margin-bottom: 10px;
          &:first-child {
            margin-top: 0;
          }
          &:hover {
            color: @link-color;
            cursor: pointer;
          }
        }
      }
    }

    .btn {
      white-space: normal;
    }
    .cradmin-legacy-multiselect2-target-submitbuttons {
      margin-top: 5px;
    }
  }

  .cradmin-legacy-multiselect2-target-with-items-content {
    display: none;
  }
  .cradmin-legacy-multiselect2-target-without-items-content {
    display: block;
  }
  .cradmin-legacy-multiselect2-target-no-items-selected {
    display: none;
    color: @text-muted;
    font-style: italic;
  }

  &.cradmin-legacy-multiselect2-target-with-items {
    .cradmin-legacy-multiselect2-target-with-items-content {
      display: block;
    }
    .cradmin-legacy-multiselect2-target-without-items-content {
      display: none;
    }
  }

  &.cradmin-legacy-multiselect2-target-empty-selection-allowed {
    .cradmin-legacy-multiselect2-target-with-items-content {
      display: block;
    }
    .cradmin-legacy-multiselect2-target-selected-items {
      display: none;
    }
    .cradmin-legacy-multiselect2-target-without-items-content {
      display: none;
    }
    .cradmin-legacy-multiselect2-target-no-items-selected {
      display: block;
    }
    &.cradmin-legacy-multiselect2-target-with-items {
      .cradmin-legacy-multiselect2-target-selected-items {
        display: block;
      }
      .cradmin-legacy-multiselect2-target-no-items-selected {
        display: none;
      }
    }
  }
}
