ul#cradmin_legacy_messages {
  margin: 0 0 10px 0;
}

body.cradmin-legacy-formview-page {
  ul#cradmin_legacy_messages {
    max-width: @cradmin-form-max-width;
  }
}

//#cradmin_legacy_messages_standalone_wrapper {
//  ul#cradmin_legacy_messages {
//    .alert {
//      font-size: @font-size-large;
//      &.alert-dismissable button.close,
//      .cradmin-legacy-alert-loadspinner {
//        top: 4px;
//      }
//    }
//  }
//}
