// Menu
@cradmin-default-theme-mainmenu-width: 200px;
@cradmin-default-theme-mainmenu-footer-stick-to-bottom: true;
@cradmin-default-theme-mainmenu-footer-margin-top: 30px;
@cradmin-default-theme-mainmenu-footer-height: 110px;
@cradmin-default-theme-mainmenu-top-transition-duration: 0.12s;

// Body and bodywrapper
@cradmin-bodybg: #e1e1e1;
@cradmin-bodycontent-bg: #E6E6E6;
