ol.cradmin-legacy-listbuilder-list {

  //
  //
  // Basic typography
  //
  //
  font-size: @font-size-small;

  h2, h3 {
    font-size: @font-size-base;
    &:first-child {
      margin-top: 0;
      padding-top: 0;
    }
  }
  p, pre, table, form, h2, h3 {
    &:last-child {
      margin-bottom: 0;
      padding-bottom: 0;
    }
  }


  //
  //
  // .cradmin-legacy-listbuilder-itemframe-defaultspacing
  //
  //

  &.cradmin-legacy-listbuilder-rowlist {
    li {
      .cradmin-legacy-listbuilder-itemframe.cradmin-legacy-listbuilder-itemframe-defaultspacing {
        margin-top: 0;
        margin-bottom: 5px;
      }
    }
  }

  &.cradmin-legacy-listbuilder-floatgridlist {
    li {
      .cradmin-legacy-listbuilder-itemframe.cradmin-legacy-listbuilder-itemframe-defaultspacing {
        margin-top: 5px;
        margin-bottom: 5px;
        @media (min-width: @screen-sm-min) {
          margin: 5px;
        }
      }
    }
  }

  &:first-child {
    &.cradmin-legacy-listbuilder-rowlist {
      li {
        &:first-child {
          .cradmin-legacy-listbuilder-itemframe.cradmin-legacy-listbuilder-itemframe-defaultspacing {
            margin-top: 0;
          }
        }
      }
    }
  }

  li {
    a.cradmin-legacy-listbuilder-itemframe-link {
      display: block;
    }
  }

}


a.cradmin-legacy-listbuilder-itemframe-link {
  color: @text-color;
  text-decoration: none;

  &:hover {
    color: @text-color;
    text-decoration: none;
  }
}
