.cradmin-legacy-loadmorepager {
  text-align: center;
  padding-top: 20px;
  padding-bottom: 20px;

  &.cradmin-legacy-loadmorepager-hidden {
    display: none;
  }
}

.cradmin-legacy-loadmorepager-target {
  opacity: 1;
  .css3-transition(opacity 0.2s ease-in-out);
  &.cradmin-legacy-loadmorepager-target-reloading-page1 {
    opacity: 0;
    button, a, input {
      display: none;
    }
  }
}
