ol.cradmin-legacy-listbuilder-list {
  li {
    .cradmin-legacy-listbuilder-itemvalue.cradmin-legacy-listbuilder-itemvalue-focusbox {
      background: @cradmin-listbuilder-itemvalue-focusbox-bg;
      padding: @cradmin-listbuilder-itemvalue-focusbox-padding;
    }
  }
}


a.cradmin-legacy-listbuilder-itemframe-link {
  .cradmin-legacy-listbuilder-itemvalue.cradmin-legacy-listbuilder-itemvalue-focusbox {
    border-bottom: 3px solid transparent;
    -webkit-tap-highlight-color: @cradmin-btn-tap-highlight-color;
    @media (min-width: @screen-sm-min) {
      .css3-transition(all @cradmin-btn-transition-duration ease-in-out);
    }
    .cradmin-legacy-listbuilder-itemvalue-titledescription {
      color: @text-color;
      .cradmin-legacy-listbuilder-itemvalue-titledescription-title {
        color: @link-color;
      }
    }
  }

  &:hover {
    .cradmin-legacy-listbuilder-itemvalue.cradmin-legacy-listbuilder-itemvalue-focusbox {
      border-color: @link-color;
      .cradmin-legacy-listbuilder-itemvalue-titledescription {
        .cradmin-legacy-listbuilder-itemvalue-titledescription-title {
          color: @link-hover-color;
          text-decoration: underline;
        }
      }
    }
  }
}
