#: Font-awesome icon map for the ``cradmin_icon`` template tag.
FONT_AWESOME = {
    "search": "fa fa-search",
    "x": "fa fa-times",
    "h1": "fa fa-header cradmin-legacy-icon-font-lg",
    "h2": "fa fa-header",
    "h3": "fa fa-header cradmin-legacy-icon-font-sm",
    "bold": "fa fa-bold",
    "italic": "fa fa-italic",
    "link": "fa fa-link",
    "list-ul": "fa fa-list-ul",
    "list-ol": "fa fa-list-ol",
    "codeblock": "fa fa-code",
    "loadspinner": "fa fa-spin fa-spinner",
    "caret-down": "fa fa-caret-down",
    "caret-up": "fa fa-caret-up",
    "pager-next-page": "fa fa-chevron-right",
    "pager-previus-page": "fa fa-chevron-left",
    "close-overlay-right-to-left": "fa fa-chevron-left",
    "select-left": "fa fa-angle-right",
    "select-right": "fa fa-angle-left",
    "multiselect2-selected": "fa fa-check",
}
