nav#cradmin_legacy_mainmenu.cradmin-legacy-menu {
  @media (min-width: @screen-md-min) {
    position: absolute;
    top: 0;
    height: 100%;
    left: 0;
    width: @cradmin-default-theme-mainmenu-width;
    overflow-y: auto;
    overflow-x: hidden;
    background: none;
    opacity: 1.0;
    .css3-transition(all @cradmin-default-theme-mainmenu-top-transition-duration ease-in-out);

    .cradmin-legacy-menu-content {
      background: none;
      padding-bottom: @cradmin-default-theme-mainmenu-footer-height;
      ul {
        &.cradmin-legacy-menu-content-footer {
          margin-top: 0;
          position: absolute;
          left: 0;
          right: 0;
          bottom: 0;
        }
      }
    }

    &.cradmin-legacy-menu-overflow-y {
      .cradmin-legacy-menu-content {
        padding-bottom: 0;
        ul.cradmin-legacy-menu-content-footer {
          margin-top: @cradmin-default-theme-mainmenu-footer-margin-top;
          position: static;
        }
      }
    }

    &.cradmin-legacy-menu-scrolling {
      opacity: 0.5;
    }
  }
}


#cradmin_legacy_mainmenu_backdrop {
  @media (min-width: @screen-md-min) {
    display: block;
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    height: 100%;
    width: @cradmin-default-theme-mainmenu-width;
    background: @cradmin-menu-bg;
  }
}
