.cradmin-acemarkdown {
  textarea.cradmin-acemarkdowntextarea {
    display: none;
  }

  &.cradmin-acemarkdown-textareavisible {
    textarea.cradmin-acemarkdowntextarea {
      display: block;
    }
  }

  .btn-toolbar {
    margin-bottom: 10px;
  }

  .ace_editor {
    font-size: @font-size-base;
    height: 400px;
    background: @cradmin-form-field-inactive-bg;
  }
}

.cradmin-legacy-form {
  .cradmin-focusfield .form-group .controls {
    .cradmin-acemarkdown.cradmin-div-styled-as-textarea {
      background: @cradmin-form-field-inactive-bg;

      .ace_editor {
        height: 100px;

        @media (min-height: 400px) {
          height: 200px;
        }
        @media (min-height: 500px) {
          height: 300px;
        }
        @media (min-height: 600px) {
          height: 400px;
        }
        @media (min-height: 800px) {
          height: 600px;
        }
      }

      &.cradmin-focus {
        background: @cradmin-form-field-focus-bg;
        .ace_editor {
          background: @cradmin-form-field-focus-bg;
        }
      }

    }
  }
}
